package oracle.forms.jdapi.demo;

import oracle.forms.jdapi.demo.util.CommandLineArgs;
import oracle.forms.jdapi.demo.util.DemoArgumentsException;
import oracle.forms.jdapi.*;

/** Forms JAVA API (JDAPI): <b>Sample 3</b> - <em>GetVersion</em><br>
 *  This sample carries out the very simple task of obtaining the version number
 *  of Forms that saved a module.
 *  However, the sample is more realistic as it introduces constructs such as
 *  command line parsing and error handling.
 *
 * @author drmills
 * @author 10-August-2001
 * @version 1.0
 */
public class GetVersion
{
  static String s_sample_name    = "GetVersion";
  static String s_sample_version = "1.0";

  /**
   * Everything in this class is done in the constructor with the exception of command line parsing.<br>
   *
   * The JDAPI is also explicitly shutdown when finished with.
   *
   * @param <b>mainArgs</b> array of strings containing the arguments
   * @return no return value
   * @exception no exceptions thrown
   */
  public GetVersion(String[] mainArgs)
  {
    // Print Header
    System.out.println(s_sample_name + " Version " + s_sample_version);

    // Here we introduce a utility class CommandLineArgs that these demos
    // use to extract name = value pairs from the command line
    // in this case only one pair is allowed MODULE=<name>
    CommandLineArgs args = new CommandLineArgs(mainArgs);
    String moduleName;



    try
    {
      //Process the command line arguments to make sure we
      //the name of the module to examine.
      moduleName = args.getValue("MODULE",null);

      if (moduleName == null)
      {
        //So, no named argument was supplied, lets try a positional Argument
        moduleName = args.getValueByPosition(1);
      }

      if (moduleName == null)
        throw new DemoArgumentsException("Module name and location must be supplied");

      System.out.println("Raw Value from getProductVersion: " + Jdapi.getProductVersion());
      
      //Format that to a recognisable Forms Version Number
      System.out.println("JDAPI Running on Forms "
                        + toVersionString(Jdapi.getProductVersion()));

      // Now we'll print out the version of Forms that last saved this
      // module.  This uses a static method in JdapiModule
      long modVersion = JdapiModule.getModulesProductVersion(moduleName);


      // Print that out in the correct format
      System.out.println(moduleName + " was created with Forms version "
                        + toVersionString(modVersion));


    }
    catch (DemoArgumentsException daex)
    {
      printError(daex.toString());
      this.printUsage();
    }
    catch (JdapiException jex)
    {
      printError(jex.toString());
    }

    // Clean up
    Jdapi.shutdown();

  }

  public static void printUsage()
  {
    System.out.println("\n" + s_sample_name + ":\n-------------------\n"
                     + "This class extracts the version number "
                     + "of Forms that last saved this module.\n"
                     + "Usage:\n------\n"
                     + "java oracle.forms.jdapi.demo.GetVersion MODULE=<name>");
  }

  private void printError(String errorString)
  {
    System.out.println(s_sample_name + " ERROR: " + errorString);
  }

  private String toVersionString(long versionNo)
  {
     return new String((versionNo/10000000) % 10 + "."
                     + (versionNo/1000000) % 10  + "."
                     + (versionNo/10000) % 100   + "."
                     + (versionNo/100) % 100     + "."
                     +  versionNo % 100);
  }


  public static void main(String[] args)
  {
    try 
    {
      GetVersion getVersion = new GetVersion(args);
    }
    catch (java.lang.ArrayIndexOutOfBoundsException aiobex)
    {
      GetVersion.printUsage();
    }  
  }

}