package oracle.forms.jdapi.demo;

import oracle.forms.jdapi.demo.util.CommandLineArgs;
import oracle.forms.jdapi.demo.util.DemoArgumentsException;
import oracle.forms.jdapi.*;

/** Forms JAVA API (JDAPI): <b>Sample 5</b> - <em>ListLibraries</em><br>
 *  In this sample we list out all of the PL/SQL Libraries that are attached to
 *  the module that is supplied as a command line arg to the class
 *  This introduces JdapiIterator for iterating through child objects
 *
 * @author drmills
 * @author 13-August-2001
 * @version 1.0
 */
public class ListLibraries
{
  static String s_sample_name    = "ListLibraries";
  static String s_sample_version = "1.0";

  /**
   * Everything in this class is done in the constructor with the exception of
   * command line parsing.
   * @param <b>args</b> arguments passed from the command line
   * @return no return value
   * @exception no exceptions thrown
   */
  public ListLibraries(String[] mainArgs)
  {
    //Print header
    System.out.println(s_sample_name + " Version " + s_sample_version);

    // use the utility class CommandLineArgs to find the name of the
    // form to process
    CommandLineArgs args = new CommandLineArgs(mainArgs);
    String moduleName;
    JdapiIterator pllIterator = null;

    try
    {
      //Process the command line arguments to make sure we
      //the name of the module to examine.
      moduleName = args.getValue("MODULE",null);

      if (moduleName == null)
      {
        //So, no named argument was supplied, lets try a positional Argument
        moduleName = args.getValueByPosition(1);
      }

      if (moduleName == null)
        throw new DemoArgumentsException("Module name and location must be supplied");

      // Before we open the module set the JDAPI Preference to raise an 
      // error if attached libraries cannot be found as that is what we are
      // interested in.  We can ignore Subclass failures though...
      Jdapi.setFailLibraryLoad(true);
      Jdapi.setFailSubclassLoad(false);

      // Open the module - assume it could be of any type at this point
      JdapiModule mod = null;
      try
      {
         mod = JdapiModule.openModule(moduleName);
      }
      catch (JdapiException attex)
      {
        // Pass the exception up if we failed to load Libraries
        throw(attex);
      }


      // At this point we have to work out the type and use the
      // correct method to get the JdapiIterator for the list of attached
      // libraries - branch on type.
      if (mod.getTypeId() == JdapiTypes.FORM_MODULE_OTID)
      {
        pllIterator = ((FormModule)mod).getAttachedLibraries();
      }
      else if (mod.getTypeId() == JdapiTypes.MENU_MODULE_OTID)
      {
        pllIterator = ((MenuModule)mod).getAttachedLibraries();
      }
      else if (mod.getTypeId() == JdapiTypes.LIBRARY_MODULE_OTID)
      {
        pllIterator = ((PlsqlModule)mod).getAttachedLibraries();
      }
      else
      {
        throw new DemoArgumentsException("Object Libraries cannot have attached Libraries");
      }

      // Now we have an Iterator object we can go through each of the elements
      // and print out the information

      if (!pllIterator.hasNext())
      {
        System.out.println(mod.getName() + "(" + mod.getClassName() + ")"
                           + " has no attached Libraries." );
      }
      else
      {
        System.out.println("Libraries attached to " +mod.getName()
                           + "(" + mod.getClassName() + ")");
        AttachedLibrary lib = null;
        while (pllIterator.hasNext())
        {
          // get the library
          lib = (AttachedLibrary)pllIterator.next();

          // print out the name and attachment info
          System.out.println("\t" + lib.getName() + " -  " +
            (String)((lib.getLibrarySource()==JdapiTypes.LISR_DATABASE_CTID)?"(Stored in Database)":lib.getLibraryLocation())
            );
        }
      }
    }
    catch (DemoArgumentsException daex)
    {
      printError(daex.toString());
      this.printUsage();
    }
    catch (JdapiException jex)
    {
      printError(jex.toString());
    }

    // Clean up
    Jdapi.shutdown();
  }


  /**
   * Simply prints an error message to the console
   * @param <b>errorString</b> the message to print
   * @return no return value
   * @exception no exceptions thrown
   */
  private void printError(String errorString)
  {
    System.out.println(s_sample_name + " ERROR: " + errorString);
  }

  /**
   * Prints out the usage for this sample
   * @return no return value
   * @exception no exceptions thrown
   */
  public static void printUsage()
  {
    System.out.println("\n" + s_sample_name + ":\n-------------------\n"
                     + "This class extracts the list of PLLs attached "
                     + "to this module.\n"
                     + "Usage:\n------\n"
                     + "java oracle.forms.jdapi.demo.ListLibraries MODULE=<name>");
  }

  public static void main(String[] args)
  {
    ListLibraries listLibraries = new ListLibraries(args);
  }
}