package oracle.forms.jdapi.demo;

import oracle.forms.jdapi.demo.util.CommandLineArgs;
import oracle.forms.jdapi.*;

/** Forms JAVA API (JDAPI): <b>Sample 4</b> - <em>SimpleForm</em><br>
 *  In this sample we create a new Form using the API and show ways
 *  of simple object subclassing and copying
 *
 * @author drmills
 * @author 10-August-2001
 * @version 1.0
 */

public class SimpleForm
{
  static String s_sample_name    = "SimpleForm";
  static String s_sample_version = "1.0";

  /**
   * Everything in this class is done in the constructor with the exception of
   ( command line parsing.<br>
   * @param <b>args</b> arguments passed from the command line
   * @return no return value
   * @exception no exceptions thrown
   */
  public SimpleForm(String[] mainArgs)
  {
    //Print header
    System.out.println(s_sample_name + " Version " + s_sample_version);

    // Parse the command line for the name of the new form to create
    // We'll just assume positional arguments are being used in this case
    // If no Form name is supplied then we'll call the form simpleform_output.fmb
    // and put it into the working directory
    CommandLineArgs args = new CommandLineArgs(mainArgs);

    String moduleName = args.getValueByPosition(1);

    if (moduleName == null)
      moduleName = new String(System.getProperty("user.dir") + System.getProperty("file.separator") + "simpleform_output.fmb");
      System.out.println(moduleName);


    // Start the JDAPI stuff - we do this in a try{} catch block
    // so we can handle any exceptions in a graceful manner
    try
    {
      // Create a new form with the name as SIMPLEFORM
      FormModule form = new FormModule("SIMPLEFORM");

      // Create a canvas in the Form we do this by calling the
      // Canvas constructor passing the module handle and a name for the Canvas
      Canvas canvas1 = new Canvas(form,"CANVAS1");

      // Set the background and foreground colours of this canvas to an obvious
      // colour.
      // Note the colours used here are in the context of the Forms Color
      // palette not a Java colour
      // for background coluor use the simple wrapper method
      canvas1.setBackColor("cyan");

      // For Foreground do it the hard way using the generic set method with the
      // correct property ID
      // Both ways are valid - obviously the first is simpler and clearer
      canvas1.setStringProperty(JdapiTypes.FOREGROUND_COLOR_PTID, "darkred");

      // Now create a block again the parent is the Form
      Block block1 = new Block(form, "BLOCK1");

      // Add a button into the block
      Item button1 = new Item(block1, "BUTTON1");

      // This is initially a normal text Item so change it to a button
      // and give it a label
      button1.setItemType(JdapiTypes.ITTY_PB_CTID);
      button1.setLabel("Button 1");

      // Now place the button on CANVAS1 and set it's size and position
      // We could define the canvas by name or by object we'll use by object as
      // we have the canvas1 object handy already
      button1.setCanvasObject(canvas1);

      //Size and position the button - this uses the current coordinate system
      //in this case - the default Points
      button1.setWidth(100);
      button1.setHeight(14);
      button1.setXPosition(20);
      button1.setYPosition(20);

      // Create a "subclass" of the button but give it a different y pos and label
      // if you look at the created form you'll see that the non default properties
      // are all inherited from button1 apart from the two that we override below
      Item button2 = new Item(block1,"BUTTON2",button1);
      button2.setLabel("Subclassed from Button 1");
      button2.setYPosition(40);

      // Create a "copy" of the button using clone() and give it a
      // different y pos and label.
      // If you look at the created form you'll see that this is not linked
      // to the first button at all it was just created with the same attributes
      Item button3 = button1.clone(block1,"BUTTON3");
      button3.setLabel("Copied from Button 1");
      button3.setYPosition(60);

      // And save the Form so we can look at it in the builder to prove this
      // all worked
      form.save(moduleName);
      System.out.println("Form saved as " + moduleName);

    }
    catch (JdapiException jex)
    {
      printError(jex.toString());
    }

    //clean up
    Jdapi.shutdown();

  }

  private void printError(String errorString)
  {
    System.out.println(s_sample_name + " ERROR: " + errorString);
  }

  public static void main(String[] args)
  {
    SimpleForm simpleForm = new SimpleForm(args);
  }
}