package oracle.forms.jdapi.demo;

import oracle.forms.jdapi.demo.util.CommandLineArgs;
import oracle.forms.jdapi.demo.util.DemoArgumentsException;
import oracle.forms.jdapi.*;

/** Forms JAVA API (JDAPI): <b>Sample 6</b> - <em>Traverse</em><br>
 *  In this sample we list out all of the objects that are within 
 *  the module that is supplied as a command line arg to the class
 *  The name of the object (if it has one) and the Class (in JDAPI terms) 
 *  is printed out in a simple list
 *  This introduces JdapiIterator for iterating through child objects
 *
 * @author drmills
 * @author 13-August-2001
 * @version 1.0
 */
public class Traverse
{
  static String s_sample_name    = "Traverse";
  static String s_sample_version = "1.0";

  /**
   * The traverse constructor simply parses the command line
   * and opens the module it then calls recurseObjects to
   * work through the module
   * @param <b>args</b> arguments passed from the command line
   * @return no return value
   * @exception no exceptions thrown
   */
  public Traverse(String[] mainArgs)
  {
    //Print header
    System.out.println(s_sample_name + " Version " + s_sample_version);

    // use the utility class CommandLineArgs to find the name of the
    // form to process
    CommandLineArgs args = new CommandLineArgs(mainArgs);
    String moduleName;

    try
    {
      //Process the command line arguments to make sure we
      //the name of the module to examine.
      moduleName = args.getValue("MODULE",null);

      if (moduleName == null)
      {
        //So, no named argument was supplied, lets try a positional Argument
        moduleName = args.getValueByPosition(1);
      }

      if (moduleName == null)
        throw new DemoArgumentsException("Module name and location must be supplied");


      // Open the module - assume it could be of any type at this point
      JdapiModule mod = JdapiModule.openModule(moduleName);

      recurseObject((JdapiObject)mod);

    }
    catch (DemoArgumentsException daex)
    {
      printError(daex.toString());
      this.printUsage();
    }
    catch (JdapiException jex)
    {
      printError(jex.toString());
    }

    // Clean up
    Jdapi.shutdown();
  }

  /**
   * Given a JDAPI object recurseObject inspects it
   * for any children using it's metadata object and
   * drills down through that
   * @param <b>obj</b> JdapiObject to inspect
   * @return no return value
   * @exception JdapiException
   */
  private void recurseObject(JdapiObject obj) throws JdapiException
  {
    // First print out the name of this object
    System.out.println("Found object " + obj.getName() + " " + obj.getClass());

    // use the passed object's metadata object
    // to obtain its possible child object metaproperties
    JdapiMetaObject meta = obj.getJdapiMetaObject();
    JdapiIterator childProps = meta.getChildObjectMetaProperties();

    // Iterate through those property types
    while(childProps.hasNext())
    {
       JdapiMetaProperty mprop = (JdapiMetaProperty)childProps.next();

       //now pull out every instance of this object property
       JdapiIterator childObjects = obj.getChildObjectProperty(mprop.getPropertyId());
       if (childObjects != null)
       {
         while (childObjects.hasNext())
         {
            // finally, recurse
            recurseObject((JdapiObject)(childObjects.next()));
         }
      }
    }
  }


  /**
   * Simply prints an error message to the console
   * @param <b>errorString</b> the message to print
   * @return no return value
   * @exception no exceptions thrown
   */
  private void printError(String errorString)
  {
    System.out.println(s_sample_name + " ERROR: " + errorString);
  }

  /**
   * Prints out the usage for this sample
   * @return no return value
   * @exception no exceptions thrown
   */
  public static void printUsage()
  {
    System.out.println("\n" + s_sample_name + ":\n-------------------\n"
                     + "This class traverses the entire object model of "
                     + "a modulemodule.\n"
                     + "Usage:\n------\n"
                     + "java oracle.forms.jdapi.demo.Traverse MODULE=<name>");
  }

  public static void main(String[] args)
  {
    Traverse traverse = new Traverse(args);
  }
}