package oracle.forms.jdapi.demo.util;

import java.util.Hashtable;
import java.util.Enumeration;
import java.lang.StringBuffer;

/** Forms JAVA API (JDAPI): <u>Utilities</u> - <em>CommandLineArgs</em><br>
 *  A helper class that is used to parse an array of strings (such as a command
 *  line), into a series of String = Value pairs
 *
 * @author drmills
 * @author 09-August-2001
 * @version 1.0
 */
public class CommandLineArgs
{
  static String s_posPrefix        = "POSITIONAL-ARG-";
  static String s_flagPrefix       = "FLAG";

  Hashtable m_argList              = new Hashtable();
  int       m_positionalHighwater  = 0;
  boolean   m_positionalAfterNamed = false;

  /**
   * Accepts an array of Strings containing String=Value pairs
   * to make it useful
   * If an argument comes in with no "=" then it is assigned
   * to an entry in the hashtable called POSITIONAL_ARG_<n>
   * where <n> is a 1 based index, unless the argument begins
   * with a "-" in which case it is deemed to be a flag and is assigined
   * to an entry called FLAG-<flag> and the next token in the input
   * array becomes the value of that flag if it itself is not a flag
   * or name value pair
   *
   * @param <b>args</b> Array of Strings - usually from Main()
   * @return no return value
   * @exception no exceptions thrown
   */
  public CommandLineArgs(String[] args)
  {
    String  name;
    String  value;
    boolean namedArgFound = false;

    for (int i = 0; i < args.length;i++)
    {
      int off = args[i].indexOf('=');
      if (off == -1)
      {
        if (args[i].startsWith("-"))
        {
          name = new String(s_flagPrefix + args[i]);
          // Assume it's a flag - check the next value to see if that could
          // be a value for this flag
          if ((args[i+1].indexOf('=') == -1) && !(args[i+1].startsWith("-")))
          {
            value = args[++i];
          }
          else
          {
            // If the next argument is nother flag etc then we assume this was
            // a lone switch and just set it's value to itself
            value = args[i];
          }
        }
        else
        {
          // Treat it as positional
          name = new String(s_posPrefix + ++m_positionalHighwater);
          value = args[i];
          // Set the variable which indicates a mix and match of types with
          // positional after named on the command line
          if (namedArgFound)
            m_positionalAfterNamed = true;
        }
      }
      else
      {
        // Conventional Name=value pair
        namedArgFound = true;
        name   = args[i].substring(0,off).toUpperCase();
        value  = args[i].substring(off+1);
      }
      m_argList.put(name, value);
    }
  }

  /**
   * Searches the argument list for the named argument and returns it's
   * value.  If the argmument was not supplied then the default value will
   * be returned instead
   * @param <b>argName</b> the name of the argument to get the value for e.g. SID
   * @param <b>defaultValue</b> the value to return if a value does not exist for the specified argument.
   * @returns String value of the required argument.
   */
  public String getValue(String argName, String defaultValue)
  {
    String returnValue = (String)m_argList.get(argName.toUpperCase());
    if (returnValue == null)
    {
      returnValue = defaultValue;
    }
    return returnValue;

  }
  /**
   * Searches the argument list for the named argument and returns it's
   * value.  If the argmument was not supplied then the default value will
   * be returned instead. Ths version of get returns the value a uppercase so it's
   * ideal for values that are used as flags
   * @param <b>argName</b> the name of the argument to get the value for e.g. SID
   * @param <b>defaultValue</b> the value to return if a value does not exist for the specified argument.
   * @returns Uppercase String value of the required argument.
   */
  public String getUppercaseValue(String argName, String defaultValue)
  {
    String returnValue = getValue(argName, defaultValue);
    if (returnValue != null)
    {
      return returnValue.toUpperCase();
    }
    else
    {
      return null;
    }
  }

  /**
   * Returns the number of positional Args or zero
   * @returns number of positional arguments.
   */
  public int getPositionalArgCount()
  {
    return m_positionalHighwater;
  }

  /**
   * Returns the value of positional argument
   * @returns string value.
   */
  public String getValueByPosition(int pos)
  {
    return (String)m_argList.get(s_posPrefix + pos);
  }

  /**
   * Indicates if positional Args are present on the command line after
   * named pair arguments
   * @returns true or false.
   */
  public boolean isArglistMixedUp()
  {
    return m_positionalAfterNamed;
  }

  /**
   * Indicates if a flag (e.g. -d) was set
   * @returns true or false.
   */
  public boolean isFlagSet(String flag)
  {
    return m_argList.containsKey(s_flagPrefix + flag);
  }

  /**
   * Returns the value of flagged argument (e.g. -d c:\temp would return c:\temp)
   * this could return the flag itself if no corresponding value to
   * the flag was present on the command line
   * @returns string value.
   */
  public String getValueByFlag(String flag)
  {
    return (String)m_argList.get(s_flagPrefix + flag);
  }


  /**
   * For debugging - dumps out the arguments
   * @returns delimited string
   */
  public String toString()
  {
    StringBuffer buff = new StringBuffer("Command Line Args:");
    buff.append("\n\t" + m_argList.toString());
    return buff.toString();
  }
}