
// Copyright (c) 2000 Oracle Corp
package oracle.forms.demos.jspell;

import oracle.forms.ui.*;
import oracle.forms.handler.IHandler;
import oracle.forms.engine.Main;
import oracle.forms.properties.ID;
import com.wallstreetwise.core.gui.*;
import com.wallstreetwise.app.jspell.domain.*;
import com.wallstreetwise.app.jspell.domain.net.*;
import com.wallstreetwise.app.jspell.gui.*;

import java.awt.*;
import java.net.URL;
import oracle.forms.handler.IHandler;
import oracle.ewt.lwAWT.lwText.*;

/**
 * A PJC class used to integrate a Multi Line Text area in a Form with the JSpell Java Spellchecker
 * <P>
 * @author Duncan Mills
 */
public class SpellCheckTextArea extends VTextArea implements JSpellTextComponent{

  /**
   * The property that we'll use to trigger a spellcheck
   */
  public final static ID SPELLCHECK        = ID.registerProperty("SPELLCHECK");

  /**
   * The property that we'll use to set the Spellcheck Dictionary type SOCKET or SERVLET
   */
  public final static ID DICTIONARY_TYPE    = ID.registerProperty("DICTIONARY_TYPE");

  /**
   * The property that we'll use to set the Spellcheck Socket m/c
   */
  public final static ID DICTIONARY_SERVER  = ID.registerProperty("DICTIONARY_SERVER");

  /**
   * The property that we'll use to set the Spellcheck Socket Port
   */
  public final static ID DICTIONARY_PORT  = ID.registerProperty("DICTIONARY_PORT");


  /**
   * The property that we'll use to set batch Size for checking
   */
  public final static ID BATCH_SIZE  = ID.registerProperty("BATCH_SIZE");

  /**
   * The property that we'll use to set max no of suggestions
   */
  public final static ID MAX_SUGGESTIONS  = ID.registerProperty("MAX_SUGGESTIONS");

  /**
   * The JSpell Property
   */
  public final static ID IGNORE_UPPER  = ID.registerProperty("IGNORE_UPPER");

  /**
   * The JSpell Property
   */
  public final static ID IGNORE_IRREGULAR  = ID.registerProperty("IGNORE_IRREGULAR");


  /**
   * The JSpell Property
   */
  public final static ID IGNORE_FIRST  = ID.registerProperty("IGNORE_FIRST");

  /**
   * The Property for disabling the Learn button
   */
  public final static ID LEARN_ENABLED  = ID.registerProperty("LEARN_ENABLED");


  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console so we can see what is going on
   * the user has selected.
   */
  private final static ID DEBUGMESSAGES = ID.registerProperty("DEBUGMESSAGES");

  private final static int USE_NONE = 0;
  private final static int USE_SOCKETS = 1;
  private final static int USE_SERVLET = 2;


  IHandler mHandler;
  Frame mAppletFrame = null;
  int mDictionaryPort;
  String mDictionaryServer = null;
  int mDictionaryType = USE_NONE;
  boolean mDebug = false;

  JSpellParser mJSpellParser = null;
  JSpellErrorHandler mJSpellHandler = null;
  JSpellDictionary mJSpellDictionary;
  int mDictionaryBatchSize = 100;
  int mDictionaryMaxSuggestions = 10;
  boolean mIgnoreUpper = true;
  boolean mIgnoreIrregular = false;
  boolean mIgnoreFirst = false;
  boolean mLearnEnabled = true;
  LWTextComponent mComp = null;


  /**
   * Implimentation of IView interface
   *
   * @param hander The context passed from forms.
   * @see IView
   */
  public void init(IHandler handler)
  {
    mHandler = handler;
    super.init(handler);
    URL codebase = handler.getCodeBase();
    mDictionaryServer = codebase.getHost();
  }


/**
 * Implimentation of IView interface
 *
 * @param pid   property to be set
 * @param value new value of the property
 * @return      true (if the property could be set), false otherwise
 * @see IView
 */
   public boolean setProperty(ID pid, Object value)
  {
    if ( pid == SPELLCHECK )
    {
      doSpellCheck();
      return true;
    }
    else if (pid == DICTIONARY_SERVER)
    {
    log("SetProperty(DICTIONARY_SERVER," + value.toString() + ")");
      if ((value!=null)&& (value instanceof String))
        mDictionaryServer = (String)value;
      return true;
    }
    else if (pid == DICTIONARY_PORT)
    {
      log("SetProperty(DICTIONARY_PORT," + value.toString() + ")");
      if (value!=null)
        mDictionaryPort = Integer.parseInt(value.toString());
      return true;
    }
    else if (pid == BATCH_SIZE)
    {
      log("SetProperty(BATCH_SIZE," + value.toString() + ")");
      if (value!=null)
        mDictionaryBatchSize = Integer.parseInt(value.toString());
      return true;
    }
    else if (pid == MAX_SUGGESTIONS)
    {
      log("SetProperty(MAX_SUGGESTIONS," + value.toString() + ")");
      if (value!=null)
        mDictionaryMaxSuggestions = Integer.parseInt(value.toString());
      return true;
    }

    else if (pid == DICTIONARY_TYPE)
    {
      log("SetProperty(DICTIONARY_TYPE," + value.toString() + ")");
      boolean rc = true;
      if ((value!=null)&& (value instanceof String))
        if (value.toString().equalsIgnoreCase("SOCKET"))
          mDictionaryType = USE_SOCKETS;
        else if (value.toString().equalsIgnoreCase("SERVLET"))
          mDictionaryType = USE_SERVLET;
        else
        {
          mDictionaryType = USE_NONE;
          System.out.println("Invalid Value for DICTIONARY_TYPE. SOCKET or SERVLET are value values");
          rc = false;
        }
      return rc;
    }
    else if (pid == IGNORE_UPPER)
    {
      log("SetProperty(IGNORE_UPPER," + value.toString() + ")");
      mIgnoreUpper = value.toString().equalsIgnoreCase("true");
      return true;
    }
    else if (pid == IGNORE_IRREGULAR)
    {
      log("SetProperty(IGNORE_IRREGULAR," + value.toString() + ")");
      mIgnoreIrregular = value.toString().equalsIgnoreCase("true");
      return true;
    }
    else if (pid == IGNORE_FIRST)
    {
      log("SetProperty(IGNORE_FIRST," + value.toString() + ")");
      mIgnoreFirst = value.toString().equalsIgnoreCase("true");
      return true;
    }
    else if (pid == LEARN_ENABLED)
    {
      log("SetProperty(LEARN_ENABLED," + value.toString() + ")");
      mLearnEnabled = value.toString().equalsIgnoreCase("true");
      return true;
    }
    else if (pid == DEBUGMESSAGES)
    {
      if (value.toString().equalsIgnoreCase("true"))
        mDebug = true;
      else
        mDebug = false;
      return true;
    }
    else
    {
      return super.setProperty(pid, value);
    }
  }

  // Implementations of the JSpell interface

  public String getText()
  {
    return (String)this.getProperty(ID.VALUE);
  }

  public void setText(String parm1)
  {
    this.setProperty(ID.VALUE,parm1);
  }

  public void select(int parm1, int parm2)
  {
    if (mComp == null)
      mComp = (LWTextComponent)this.getComponent(0);
    mComp.setSelectionStart(parm1);
    mComp.setSelectionStart(parm2);
  }

  public void deselect()
  {
    if (mComp == null)
      mComp = (LWTextComponent)this.getComponent(0);
    mComp.setSelectionStart(0);
    mComp.setSelectionStart(0);
  }


  public Frame getFrame()
  {
   if (mAppletFrame == null)
   {
     Component frm = this;
     while(frm != null && !(frm instanceof Frame))
     {
       frm = frm.getParent();
       if (frm == null)
        System.out.println("Error Component has no parent");
     }
     mAppletFrame = (Frame)frm;
   }
   return mAppletFrame;
  }

  public TextComponent getTextComponent()
  {
    return (TextComponent)this.getComponent(0);
  }

  private void doSpellCheck()
  {
    log("Beginning SpellCheck");
    if ((mDictionaryType == USE_NONE) || (mDictionaryServer == null))
      System.out.println("Error: No Dictionary Defined for SpellCheck");
    else
    {
      if (mDictionaryType == USE_SOCKETS)
      {
        JSpellDictionarySocket socketDict = (JSpellDictionarySocket)mJSpellDictionary;
        log("Spelling using Sockets: Server=" + mDictionaryServer + " Port=" + mDictionaryPort);

        //If a socket is already open, close it.
        if (socketDict != null)
          socketDict.close();

        //Establish a new Dictionary
        socketDict = new JSpellDictionarySocket();

        //Default the port of not set
        if (mDictionaryPort == 0)
          mDictionaryPort = 5317;

        //Set the server and port for the new Dictionary then check connection
        socketDict.setHost(mDictionaryServer);
        socketDict.setPort(mDictionaryPort);
        try {
              socketDict.open();
              log("Connection to Server established");
            } catch (JSpellException eSocket)
              {
                System.out.println("Error unable to establish Spelling Server Socket connection to " + mDictionaryServer + ":" + mDictionaryPort);
                eSocket.printStackTrace();
              }
        mJSpellDictionary = (JSpellDictionary)socketDict;

      }
      else
      {
       JSpellDictionaryServlet servletDict = new JSpellDictionaryServlet();
       String servletURL = null;

       if (mDictionaryPort == 0)
          mDictionaryPort = 80;

       if(mDictionaryPort==80)
         servletURL = "http://" + mDictionaryServer + "/servlet/jspell";
       else
         servletURL = "http://" + mDictionaryServer + ":" + mDictionaryPort + "/servlet/jspell";

       servletDict.setURL(servletURL);

       try {
            servletDict.open();
            log("Connection to Server established");
           } catch (JSpellException eServlet)
              {
                System.out.println("Error unable to establish Spelling Server Socket connection to " + mDictionaryServer + ":" + mDictionaryPort);
                eServlet.printStackTrace();
              }
        mJSpellDictionary = (JSpellDictionary)servletDict;
      }

      mJSpellDictionary.setBatchSize(mDictionaryBatchSize);
      mJSpellDictionary.setLearnWords(mLearnEnabled);
      mJSpellParser = new JSpellParser(mJSpellDictionary);
      mJSpellParser.setIgnoreFirstCaps(mIgnoreFirst);
      mJSpellParser.setIgnoreIrregularCaps(mIgnoreIrregular);
      mJSpellParser.setIgnoreUpper(mIgnoreUpper);
      mJSpellHandler=new JSpellErrorHandler(mJSpellParser,this);
      mJSpellHandler.setFrame(this.getFrame());
      mJSpellHandler.init();
      log("JSpell initialised");

      if(!mJSpellHandler.getBusy())
      {
        log("Spellchecking now");
        mJSpellHandler.check();
      }
      else
      {
        log("JSPELL is busy, can't spellcheck");
      }
    }
  }

  public void log(String msg)
  {
    if(mDebug)
      System.out.println(getClass().getName() + ": " + msg);
  }


}

