/*
  * @author                        : Frank Nimphius
  * @Version                       : 1.0
  *
  * Development Environment        : Oracle9i JDeveloper
  * Name of the File               : timeoutPJC.java
  * 
  * Modification Log
  * ==========================================================================
  * When                  Who               Change
  * 
  * September - 2003      Frank Nimphius    initial
  * 
  * Forms Pluggable Java Component that intercepts mouse operations and key 
  * events to reset a timer. This PJC is supposed to be used in Forms applications 
  * that need to perform some security related activities after a pre-defined
  * period of user inacticity. When starting the timer, a separate thread is 
  * spawned that frequently checks for the difference between the current client 
  * time and the last time of user interaction. If the time difference exceeds a 
  * pre-defined tolerance value, a custom event is raised to Forms for the developer
  * to handle user inactivity. 
  * 
  * To handle the case where to many user action recordings hava a performance 
  * impact to the application (e.g. refresh problems) it is possible to switch 
  * off specific events from being captured. 
  * 
*/

package oracle.forms.demos;

import java.awt.Component;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import java.text.SimpleDateFormat;

import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;

import oracle.forms.engine.Main;
import oracle.forms.handler.IHandler;
import oracle.forms.properties.ID;
import oracle.forms.ui.CustomEvent;
import oracle.forms.ui.VBean;

public class TimeoutPJC extends VBean
{
  //**********************
  // FORMS STATIC IDs
  //**********************

  /**
   * ENABLE_DEBUGGING enables log messages to be written to the console. Valid 
   * arguments are true and false
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'ENABLE_DEBUGGING','true');</code>
   */
  protected static final ID pEnableDebugging   = ID.registerProperty("ENABLE_DEBUGGING");

  /**
   * TIMER_SLEEP_TIME defines the frequency in which the Java thread checks for 
   * user actions. The default frequency is 1 minute.The value is specified as a
   * integer value representing the sleep time in seconds
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'TIMER_SLEEP_TIME','60');</code>
   */
  protected static final ID pSleepTime        = ID.registerProperty("TIMER_SLEEP_TIME");

  /**
   * START_TIMER starts the timer thread. The thread returns a custom event to
   * Forms when the user inactivity exceeds the value defined the argument. The
   * argument is specified as a number representing the expiry time in minutes
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'START_TIMER','60');</code>
   */
  protected static final ID pStartTimer        = ID.registerProperty("START_TIMER");

  /**
   * END_TIMER stops the timer thread. 
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'END_TIMER');</code>
   */
  protected static final ID pEndTimer         = ID.registerProperty("END_TIMER");
  

   /**
   * Allows to determine the granularity of events that record user actions. Valid 
   * values are 'all' [default] for all mouse and key events to record user actions, 
   * 'click' for all mouse clicks to record actions, 'enter' to record user actions on 
   * the mouse entering the Forms Applet, 'exit' to record actions when the Mouse 
   * leaves the Forms Java Applet and 'move' to record actions when the mouse is 
   * moved. Use 'key' to enable all key actions for record user actions. Focus Events
   * are recorded using 'focus' as an argument. Note that arguments can be concatenated
   * by '|'
   * <p><b>Note:</b> Key events are not captured on text items and other data entry
   * widgets except for control keys like ctrl, shift, tab, blank etc. This is caused 
   * by the underlying EWT classes intercepting these actions, not making them available 
   * to other layers of a Java form.</p> 
   * This feature is added in case that resetting the mouse on all mouse actions 
   * causes performance issues on the client. The default behavior is to reset 
   * the client on all mouse events. 
   *
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'RECORDING_EVENTS','all');</code>
   * 
   * To reset the timer on a many mouse events use the following syntax
   * 
   * <p><b>Forms Example:</b></p>
   * <code>set_custom_property('BeanArea',1,'RECORDING_EVENTS','enter|exit|click|key');</code>
   */
  protected static final ID pRecordingEvents = ID.registerProperty("RECORDING_EVENTS");

  /**
   * Custom Item Event raised when maximum inactivity time is exceeded. Note 
   * The event is caught in a when-custom-item event trigger on the Bean Container
   * 
   * <p><b>Forms Example:</b></p>
   * <p><code></p>
   * <p>...</p>
   * <p>IF (eventName='MAX_INACTIVITY_EXCEEDED') THEN</p>
   * <p>&nbsp;&nbsp;...handle expiry ... </p>
   * <p></code></p>
   */
  public static final ID ceMaxInactivityExceeded           = ID.registerProperty("MAX_INACTIVITY_EXCEEDED");
  
  /* 
   * Variables 
   */
  private                   IHandler mHandler     = null;   // Forms Handler
  private Main              formsMain             = null;   // Forms main class
  private boolean           debug_enabled         = false;
  private int               sleep_time            = 60000;  // frequency to check
                                                    // user actions      
  private long              max_inactivity;
  private long              last_user_action;
  private boolean           timerStopped          = true;
  private boolean           timerExists           = false;
  private Thread            t                     = null;
  private String            this_instance_name    = this.getDefaultName();
  private SimpleDateFormat  sdf                   = new SimpleDateFormat("dd-MMM-yyyy, hh:mm:ss a");


  private boolean           record_on_mouse_move    = true;
  private boolean           record_on_mouse_click   = true;
  private boolean           record_on_mouse_enter   = true;
  private boolean           record_on_mouse_exit    = true;
  private boolean           record_on_all_keys      = true;
  private boolean           record_on_all_events    = true;
  private boolean           record_on_focus_events  = true;
  
  // default constructor
  public TimeoutPJC()
  {
  }

  public final void init(IHandler handler)
  {
    mHandler = handler;
    super.init(handler);

    // getting the Forms Main class    
    formsMain  =  (Main) mHandler.getApplet();

    //******************************************************************
    //             ADDING MOUSE AND KEY LISTENERS TO FORMS
    //******************************************************************
      // Forms main returns desktop focus gain and lost
        _addKeyListeners(formsMain);
        _addFocusListeners(formsMain.getDesktop().getDesktopComponent());
      // the glass mouse grab provider is the proxy component for all 
      // mouse events on Forms main
        _addMouseListeners(formsMain.getProxyComponent());
    //******************************************************************
  }

    // add focus listeners to all components passed as an argument
    
    public void _addFocusListeners (Component c)
    {
      c.addFocusListener(new FocusListener(){
        public void focusLost(FocusEvent fe)
        {
          if(record_on_focus_events || record_on_all_events){
            log_activity();
          }
        }

        public void focusGained(FocusEvent fe)
        {
          if(record_on_focus_events || record_on_all_events){
            log_activity();
          }
        }
      });
    }

    // add mouse listener to all components passed as an argument
    public void _addMouseListeners(Component c){
    
      c.addMouseMotionListener(new MouseMotionListener(){

      public void mouseMoved(MouseEvent me)
      {
        if(record_on_mouse_move || record_on_all_events){
            log_activity();
        }
      }

      public void mouseDragged(MouseEvent me)
      {
        // drag and drop is not supported in Forms. Therefore the
        // drag event is useless to capture
        //log_activity();
      }
    });  

    c.addMouseListener(new MouseListener(){
     
        public void mouseExited(MouseEvent me)
        {
          if(record_on_mouse_exit || record_on_all_events){
            log_activity();
          }
        }
        public void mouseEntered(MouseEvent me)
        {
          if(record_on_mouse_enter || record_on_all_events){
            log_activity();
          }
        }

        public void mousePressed(MouseEvent me)
        {
          if(record_on_mouse_click || record_on_all_events){
            log_activity();
          }
        }

        public void mouseReleased(MouseEvent me)
        {
          if(record_on_mouse_click || record_on_all_events){
            log_activity();
          }
        }

        public void mouseClicked(MouseEvent me)
        {
          if(record_on_mouse_click || record_on_all_events){
            log_activity();
          }
        }
    });
  }

  // add key listeners to all components passed as an argument
  public void _addKeyListeners(Component c)
  {
    
        c.addKeyListener(new KeyListener(){

        public void keyReleased(KeyEvent ke)
        {
          if(record_on_all_keys || record_on_all_events){
            log_activity();
          }
        }

        public void keyPressed(KeyEvent ke)
        {
          if(record_on_all_keys || record_on_all_events){
            log_activity();
          }
        }

        public void keyTyped(KeyEvent ke)
        {
          if(record_on_all_keys || record_on_all_events){
            log_activity();
          }
        }
    });

  }

  //*******************************
  //* SET FORMS PROPERTIES 
  //*******************************
  
  public boolean setProperty(ID _ID, Object _args)
  {

   /*=========================
    * Enable Debugging
    * ========================*/
    if(_ID==pEnableDebugging)
    {
      if (_args!=null && ((String)_args).length()!=0)
      {
        if ("TRUE".equalsIgnoreCase((String)_args))
        {
          debug_enabled = true;
        }
        else
        {
          debug_enabled = false;
        }
      }
      return true;
    }
    
   /* =========================
    * Timer Frequency
    * ========================*/
    else if (_ID == pSleepTime)
    {
      if (_args != null)
        {
          try{
              sleep_time = new Integer((String)_args).intValue()*1000;
               write_message("Setting sleep intervall to "+ sleep_time/1000 +" seconds");
          }
          catch (NumberFormatException nfe)
          {
            write_message("Error in setting sleep interval - invalid syntax for sleep "+ 
                          "time. Value must be provided in seconds and of type "+
                          "integer");
          }
        }
        return true;
    }

    /*===========================
     * START TIMER 
     *==========================*/

    else if (_ID == pStartTimer)
      {
        if (_args != null)
        {
          int mx_inactivity = 0;
          try{
              mx_inactivity = new Integer((String)_args).intValue()*60*1000;
              write_message("Starting timer ... with "+ mx_inactivity +" minute(s)max. inactivity");
              startTimer(mx_inactivity);
          }
          catch (NumberFormatException nfe)
          {
            write_message("Error in starting timer - invalid syntax for max "+ 
                          "inactivity. Value must be in minutes and of type "+
                          "integer");
          }
        }
        return true;
      }

    /*===========================
     * RECORD USER ACTIONS
     *==========================*/

      if (_ID==pRecordingEvents)
      {
        // check if arguments are passed
        if (((String)_args).length()>0)
        {
          write_message("Setting events to record according to parameters: "+(String)_args);
          // tokenize argument String allowing "|" and " " as delimiters
          StringTokenizer st = new StringTokenizer((String)_args,"| ");

          if (st.countTokens() >0)
          {
            // set all event flags to false
            record_on_mouse_move    = false;
            record_on_mouse_click   = false;
            record_on_mouse_enter   = false;
            record_on_mouse_exit    = false;
            record_on_all_keys      = false;
            record_on_all_events    = false;
            record_on_focus_events  = false;

            while(st.hasMoreTokens())
            {
              String s = st.nextToken();

              if ("ENTER".equalsIgnoreCase(s))
              {
                write_message("Enabling mouse enter events ...");
                record_on_mouse_enter   = true;
              }
              else if("EXIT".equalsIgnoreCase(s))
              {
                write_message("Enabling mouse exit events ...");
                record_on_mouse_exit    = true;
              }
              else if ("MOVE".equalsIgnoreCase(s))
              { 
                write_message("Enabling mouse move events ...");
                record_on_mouse_move    = true;
              }
              else if("KEY".equalsIgnoreCase(s))
              {
                  write_message("Enabling key events ...");
                  record_on_all_keys      = true;
              }
              else if("CLICK".equalsIgnoreCase(s))
              {
                write_message("Enabling mouse click events ...");
                record_on_mouse_click   = true;
              }
              else if ("ALL".equalsIgnoreCase(s))
              {
                write_message("Enabling all events ...");
                record_on_all_events = true;
              }
              else if ("FOCUS".equalsIgnoreCase(s))
              {
                write_message("Enabling focus events ...");
                record_on_focus_events = true;
              }
              else
              {
                write_message("No valid argument when calling RECORDING_EVENTS. "+
                              "Statement ignored and all events activated for recording");

                record_on_all_events = true;
              }
            }
            return true;
          }
          else
          {
            write_message("No valid argument in call to RECORDING_EVENTS. "+
                          "Statement ignored");
            return true;
          }
        }
        else
        {
          write_message("No argument in call to RECORDING_EVENTS. Statement ignored");
          return true;
        }
      }

    /*===========================
     * END TIMER 
     *==========================*/

      else if (_ID ==pEndTimer)
      {
        write_message("Stopping timer thread ...");
        stopTimer();
        return true;
      }
     
    
      else if(_ID==pEnableDebugging)
      {
        if (_args!= null)
      {
        // enable debugging
        debug_enabled=true;
        write_message("Enabled debug messages ...");
      }
      else
      {
        // disable debugging
        debug_enabled=false;
        write_message("Disabled debug messages ...");
      }
      return true;
    }
    
    return super.setProperty(_ID, _args);
  }

  private void startTimer(int _max_inactivity)
  {
    // get start time
    if (!timerExists){
      last_user_action   = Calendar.getInstance().getTime().getTime();
      max_inactivity     = _max_inactivity;

      // debug
      write_message("Start time = "+sdf.format(new Date(last_user_action)));
      
      // timer is allowed to run
      timerStopped  = false;
      
      // create timer thread
      t = new Thread()
      {
        public void run()
        {
          // only one timer allowed per session
          timerExists = true;
          
          while (!timerStopped && !isInterrupted())
          {

            // compare last user action with current time and raise event 
            // when max allowed time is exceeded
            long current_time = Calendar.getInstance().getTime().getTime();

            // debug
            write_message("Current time = "+sdf.format(new Date(current_time)));
            write_message("Last user action time  = "+sdf.format(new Date(last_user_action)));
            write_message("Max allowed inactivity in Seconds = "+max_inactivity/1000);
            
            if ((current_time - last_user_action) < (max_inactivity))
            {

              write_message("Timer not expired ...");
              
              // zzzZZZzzzZZZzz
              try
              {
                t.sleep(sleep_time); //msec
              }
              catch (InterruptedException ie)
              {
                // ignore
              }
            }
            else
            {
              write_message("Timer expired ...");
              try{
                  // raise custom Forms event
                  CustomEvent ce = new CustomEvent(mHandler,ceMaxInactivityExceeded);
                  dispatchCustomEvent(ce);
              }
              catch (Exception e)
              {
                  write_message("Error occured when raising custom event = e.getMessage()");
              }
              
              // stop timer
              timerStopped          = true;
              // destroy timer thread
              t.interrupt();
            }
          }

          // interrupt thread when Forms calls out to stop the thread. In this
          // case the timerStopped flag is set but the isInterrupted method does
          // not return a true value
          
          if (timerStopped && !isInterrupted())
          {
            t.interrupt();
          }
          
          timerExists           = false;
        }
      };
      t.start();
    }  
  }
  
  private void stopTimer ()
  {
    timerStopped          = true;
  }

  /**
   * Central method to handle mouse and key actions for this timeoutPJC
   */
  private void log_activity()
  {
    last_user_action = Calendar.getInstance().getTime().getTime();
    
    write_message("Last user action = "+ sdf.format(new Date(last_user_action))); 
  }
  
  private void write_message(String st)
  {
    if (debug_enabled)
    {
      System.out.println("DEBUG MESSAGE "+this_instance_name+": "+st);
    }
  }

  public void destroy()
  {
    // TODO:  Override this oracle.forms.ui.VBean method
    write_message("The Bean instance holding the TimeoutPJC was stopped and the "+
                  " timeout functionality is no longer avaiable.");
    super.destroy();
  }
}